import time
from aioredlock.errors import LockError
import asyncio


async def redis_get_lock_with_timeout(lock_manager, resource, timeout=30, lock_timeout=None, sleep_second=0.1):
    """
    获取锁

    Args:

        lock_manager (Aioredlock): aioredlock的lock_manager

        resource (str): name

        timeout (int): 单位/s, 获取锁的超时时间

        lock_timeout int(s): 单位/s, 锁的持有时间

        sleep_second  (float): 获取失败之后的睡眠时间
    """
    now = int(time.time())
    end = now + timeout
    while 1:
        try:
            lock = await lock_manager.lock(resource,
                                           lock_timeout=lock_timeout)
            return lock
        except LockError:
            now = int(time.time())
            if now >= end:
                raise
            await asyncio.sleep(sleep_second)



import typing
import time


class AssertMixin(object):
    """
    用于单元测试
    """

    def assert_record_and_map_equal(self, record: typing.Union[object, dict], pairs: dict):
        """
        从 `pairs` 里面的每个key, value, `record` 必然存在且相等
        """
        for key, value in pairs.items():
            if not isinstance(record, dict):
                record_value = getattr(record, key)
            else:
                record_value = record[key]

            self.assertEqual(record_value, value, "key:{}, value:{}, record_value:{}".format(
                key, value, record_value
            ))

    def assert_wait_until(self, assert_func, timeout=20):
        """
        一直到now+timeout, 尝试判断assert_func()的返回值为True, 否则报错
        """
        now = int(time.time())
        end = now + timeout
        while 1:

            now = int(time.time())
            if now >= end:
                raise ValueError("Timeout")
            ret = assert_func()
            if ret:
                break
            time.sleep(0.05)


class RecordMixin(object):
    """
    一般用于单元测试
    """
    @classmethod
    def record_mixin_attrs_get(cls, record, dict_or_keys):
        def get_field(name):
            if isinstance(record, dict):
                return record[name]
            return getattr(record, name)
        if isinstance(dict_or_keys, dict):
            dict_or_keys = list(dict_or_keys.keys())
        dict_or_keys = list(dict_or_keys)
        d = {key: get_field(key) for key in dict_or_keys}
        return d



import setuptools

with open("readme.rst", "r") as fh:
    long_description = fh.read()

import os
name = 'cqh_util'
_dir = os.path.dirname(os.path.abspath(__file__))

init_path = os.path.join(_dir, name, '__init__.py')


def read_version():
    d = {}
    code = open(init_path).read()
    code = compile(code, '<string>', 'exec', dont_inherit=True)
    exec(code, d, d)
    return d['__version__']


version = read_version()
print("version:{}".format(version))

setuptools.setup(
    name=name,  # Replace with your own username
    version=version,
    author="chenqinghe",
    author_email="1832866299@qq.com",
    description="cqh utils function",
    long_description=long_description,
    long_description_content_type='',
    url="https://github.com/chen19901225/cqh_util",
    packages=setuptools.find_packages(),
    install_requires=[
        "gitpython",
        "jinja2",
        "aioredlock"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
    ],
    entry_points={
        "console_scripts": [
            "cqh_file_watcher=cqh_file_watcher.run:main",
        ],
    },
    python_requires='>=3.6',
    include_package_data=True
)





def user_agent_parse(param_str):
    def containswith(search_text_list):
        for search_text in search_text_list:
            if search_text in param_str:
                return True
        return False
    ret = ''
    if containswith(['Mobile']):
        ret += 'Mobile '
    else:
        ret += 'PC '

    map_list = [
        (["iPhone"], "苹果"),
        (["MicroMessenger"], "微信"),
        (["AlipayClient"], "支付宝"),

        (["QQTheme"], "QQ"),



        (["QQBrowser"], "QQ浏览器"),
        (["baiduboxapp", "baidu"], "百度"),
        (["Weibo"], "Weibo"),
        (["iqiyi"], "爱奇艺"),
        (["aweme"], "抖音"),
        (["kdtUnion_iting"], "喜马拉雅"),
        (["DingTalk"], "钉钉"),
        (["UCBrowser"], "UC"),
        (["SamsungBrowser"], "三星"),
        (["Sogou"], "搜狗"),
        #### 这里开始放手机手机型号
        (["HONORHRY"], "荣耀"),
        (["NokiaBrowser"], "诺基亚"),

        (["VivoBrowser", "vivo", "V1818T"], "Vivo"),
        (["HeyTapBrowser"], "HeyTap"),
        (["PCHM30", "OppoBrowser", "PDPM00", "PBEM00"], "OPPO"),
        (["HUAWEIEVA", "HuaweiBrowser", "huaweioem", "HUAWEI", "HWI-AL00", "HUAWEIELE", "INE-AL00", "LDN-AL00", "TAS-AN00"], "华为"),
        (["XiaoMi", "MiuiBrowser"], "小米"),
        (["HD1910"], "OnePlus"),
        (["MZBrowser"], "魅族"),
        (["Opera", "PDPM00", "PBEM00"], "Opera"),
        (["Chrome"], "谷歌"),
        (["Safari", "iPhone"], "苹果"),

    ]

    has_searched = False
    for contain_list, keyword in map_list:
        if containswith(contain_list):
            ret += keyword
            break

    if not has_searched:
        ret += "其他"

    return ret



# cqh_goto: __file__||dump_bytecode_start
from jinja2.bccache import BytecodeCache
from jinja2.utils import LRUCache
import logging
_logger = logging.getLogger(__name__)


class MemoryCache(BytecodeCache):
    '''
    添加jinaj2的redis cache
    使用的时候尽量让他成为全局变量，或者classmethod::

        class TemplateRendring(object):
            """
            A simple class to hold methods for rendering templates.
            """

            @property
            def template_env(self):
                if not hasattr(self, '_template_env'):
                    template_dirs = []
                    if self.settings.get('template_path', ''):
                        template_dirs.append(self.settings['template_path'])
                    byte_cache = self.jinja2_cache()
                    # 这个env不是全局的，那么cache有用吗？
                    env = Environment(loader=FileSystemLoader(template_dirs),
                                    auto_reload=False,
                                    bytecode_cache=byte_cache)
                    self._template_env = env
                return self._template_env

            @classmethod
            def jinja2_cache(cls):
                if not hasattr(cls, '_jinja2_cache'):
                    cls._jinja2_cache = MemoryCache(400)
                return cls._jinja2_cache


            def render_template(self, template_name, **kwargs):
                # template_dirs = []
                # if self.settings.get('template_path', ''):
                #     template_dirs.append(self.settings['template_path'])
                # env = Environment(loader=FileSystemLoader(template_dirs))
                env = self.template_env

                try:
                    """
                    # 没有bytecode_cache的时候
                    [I /tornado.general/ 200821 15:02:52 util_common:123] template_raw_render: cost 0.20970797538757324s
                    [I /tornado.general/ 200821 15:02:52 util_common:123] render_template: cost 0.27033448219299316s
                    get_template要这么久？
                    """
                    template = env.get_template(template_name)
                except TemplateNotFound:
                    raise TemplateNotFound(template_name)
                with utils.util_common_loger_context(self.logger, 'template_raw_render'):
                    # [I /tornado.general/ 200821 15:02:52 util_common:123] template_raw_render: cost 0.20970797538757324s
                    # 居然要这么久？
                    content = template.render(kwargs)
                return content

    '''

    def __init__(self, cache_size=400,
                 logger=_logger):
        self.raw_cache = LRUCache(cache_size)
        self.logger = logger

    # def get_bucket(self, environment, name, filename, source):
    #     """Return a cache bucket for the given template.  All arguments are
    #     mandatory but filename may be `None`.
    #     """
    #     self.logger.info("get_cache_key:{}, {}".format(name, filename))
    #     key = self.get_cache_key(name, filename)
    #     checksum = self.get_source_checksum(source)
    #     bucket = Bucket(environment, key, checksum)
    #     self.load_bytecode(bucket)
    #     return bucket

    def load_bytecode(self, bucket):
        try:

            code = self.raw_cache.get(bucket.key)
            self.logger.debug("load_bytecode key:{}, coder: {} ".format(bucket.key, code is None))
            self.logger.debug(self.raw_cache._mapping.keys())
        except Exception:
            raise
            code = None
        if code is not None:
            bucket.bytecode_from_string(code)
    # dump_bytecode_start

    def dump_bytecode(self, bucket):
        # args = (bucket.key, bucket.bytecode_to_string())
        try:
            # self.raw_cache.set(*args)
            self.logger.debug("dump_bytecode key:{}".format(bucket.key))
            self.raw_cache[bucket.key] = bucket.bytecode_to_string()
        except Exception:
            raise
